#!/usr/bin/python3

#	ax.purefunctions : Lib with various pure functions
#	Copyright (C) 2019-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import re

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Remove duplicates from a list

def UList(l):

	seen = set()
	l_uniq = []

	for el in l:
		if el in seen:
			continue
		seen.add(el)
		l_uniq.append(el)

	return l_uniq

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Replace all symbols that are not numbers or letters with replacer

def ReplaceNotAlfa(string, replacer, allow_in_versions=False, exceptions=set()):

	to_be_repaced = {'\s', r'\\', '/', ',', '.', '~', '*', '%', '_', '+'}

	for exception in exceptions:
		to_be_repaced.discard(exception)

	regex  = r'(?<!\d)' if allow_in_versions else r''
	regex += r'(['
	regex += r'' if '-' in exceptions else r'\-'
	regex += r''.join(to_be_repaced) + r']+)'

	return re.sub(regex, replacer, string)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Convert given num to a string of "0*[0-9]+" format to match in length with a
# string representation of a given max number, like "001" for (1, 999)

def PrettyNum(num, max_num):

	num_str = str(num)
	len_diff = len(str(max_num)) - len(num_str)

	return ('0' * len_diff + num_str if len_diff > 0 else num_str)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Divide given integer into a given number of close-to-equal integers that
# total the initial integer

def NearlyEqualShares(total, n):

	share = [total // n] * n

	for i in range(total % n):
		share[i] += 1

	return share
