#!/usr/bin/python3

#	ax.sefunctions : Lib with various functions that has side effects
#	Copyright (C) 2019-2022 Alexey Appolonov
#
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>.

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

import os
import fnmatch
from datetime    import datetime
from collections import defaultdict

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Check given directory path and try to create it if it doesn't exist

def PrepareDir(path):

	if not path:
		return None, 'Empty path'

	if path[0] == '~':
		path = os.path.expanduser(path)
	elif not os.path.isabs(path):
		path = os.path.abspath(path)

	# Creating a path if it doesn't exist
	if os.path.exists(path):
		if not os.path.isdir(path):
			return None, f'There is a file with the same path "{path}"'
	else:
		try:
			os.makedirs(path)
		except:
			return None, f'Can\'t create "{path}" dir'

	return path, ''

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Check existence of a given file or an access to that file (if optional param
# is given)

def CheckFile(filename, mode=None):

	if not filename:
		return False

	if filename[0] != '/':
		filename = os.path.join(os.getcwd(), filename)

	if (mode and not os.access(filename, mode)) or \
			(not mode and not os.path.exists(filename)):
		return False

	return True

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Recursively search for a file in a given dir

def FindFile(top_level_dir, filename):

	result = []

	for root, dirs, files in os.walk(top_level_dir):
		for f in files:
			if fnmatch.fnmatch(f, filename):
				result.append(os.path.join(root, f))

	return result

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Get current time as a "YYYY-MM-DD_hh:mm:ss" string (by default)

def DateTimeNow(date_sep='-', time_sep=':', dt_sep='_'):

	ts = datetime.now()

	return f'{ts.year}{date_sep}{ts.month:02d}{date_sep}{ts.day:02d}{dt_sep}' \
		f'{ts.hour:02d}{time_sep}{ts.minute:02d}{time_sep}{ts.second:02d}'
